@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2000-2008 MySQL AB, 2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+   Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -374,11 +373,6 @@ enum ha_base_keytype {
   update handler::auto_increment_value
 */
 #define HA_STATUS_AUTO          64
-/*
-  update the number of rows updated and deleted since last reset call.
-  handler::rows_updated, rows_deleted
-*/
-#define HA_STATUS_WRITTEN_ROWS  128
 /*
   Errorcodes given by handler functions
 
