@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2003-2006 MySQL AB
-   Use is subject to license terms.
+   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,6 +38,10 @@ const BlockName BlockNames[] = {
   ,{ "LGMAN", LGMAN }
   ,{ "PGMAN", PGMAN }
   ,{ "RESTORE", RESTORE }
+  ,{ "DBINFO", DBINFO }
+  ,{ "DBSPJ", DBSPJ }
+  ,{ "THRMAN", THRMAN }
+  ,{ "TRPMAN", TRPMAN }
 };
 
 const BlockNumber NO_OF_BLOCK_NAMES = sizeof(BlockNames) / sizeof(BlockName);
