@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2004, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2004, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,12 +17,14 @@
 
 
 #include <ndb_global.h>
+
 #include <my_base.h>
 #include <ndberror.h>
 #include <m_string.h>
 
 #include "../mgmsrv/ndb_mgmd_error.h"
 
+#include "NdbQueryBuilderImpl.hpp"
 
 typedef struct ErrorBundle {
   int code;
@@ -94,6 +96,7 @@ static const char* empty_string = "";
  * 4500 - ""
  * 4600 - ""
  * 4700 - "" Event
+ * 4800 - API, QueryBuilder
  * 5000 - Management server
  */
 
@@ -137,6 +140,7 @@ ErrorBundle ErrorCodes[] = {
     "Transaction was committed but all read information was not "
     "received due to node crash" },
   { 4119, DMEC, NR, "Simple/dirty read failed due to node failure" },
+  { 20016, DMEC, NR, "Query aborted due to node failure" },
   
   /**
    * Node shutdown
@@ -167,7 +171,7 @@ ErrorBundle ErrorCodes[] = {
    * TemporaryResourceError
    */
   { 217,  DMEC, TR, "217" },
-  { 218,  DMEC, TR, "218" },
+  { 218,  DMEC, TR, "Out of LongMessageBuffer" },
   { 219,  DMEC, TR, "219" },
   { 233,  DMEC, TR,
     "Out of operation records in transaction coordinator (increase MaxNoOfConcurrentOperations)" },
@@ -187,6 +191,7 @@ ErrorBundle ErrorCodes[] = {
   { 1217, DMEC, TR, "Out of operation records in local data manager (increase MaxNoOfLocalOperations)" },
   { 1218, DMEC, TR, "Send Buffers overloaded in NDB kernel" },
   { 1220, DMEC, TR, "REDO log files overloaded (increase FragmentLogFileSize)" },
+  { 1234, DMEC, TR, "REDO log files overloaded (increase disk hardware)" },
   { 1222, DMEC, TR, "Out of transaction markers in LQH" },
   { 4021, DMEC, TR, "Out of Send Buffer space in NDB API" },
   { 4022, DMEC, TR, "Out of Send Buffer space in NDB API" },
@@ -194,6 +199,9 @@ ErrorBundle ErrorCodes[] = {
   { 1501, DMEC, TR, "Out of undo space" },
   {  288, DMEC, TR, "Out of index operations in transaction coordinator (increase MaxNoOfConcurrentIndexOperations)" },
   {  289, DMEC, TR, "Out of transaction buffer memory in TC (increase TransactionBufferMemory)" },
+  {  780, DMEC, TR, "Too many schema transactions" },
+  {  783, DMEC, TR, "Too many schema operations" },
+  {  785, DMEC, TR, "Schema object is busy with another schema transaction" },
   {  291, DMEC, TR, "Out of scanfrag records in TC (increase MaxNoOfLocalScans)" },
 
   /**
@@ -202,16 +210,21 @@ ErrorBundle ErrorCodes[] = {
   { 623,  HA_ERR_RECORD_FILE_FULL, IS, "623" },
   { 624,  HA_ERR_RECORD_FILE_FULL, IS, "624" },
   { 625,  HA_ERR_INDEX_FILE_FULL, IS, "Out of memory in Ndb Kernel, hash index part (increase IndexMemory)" },
-  { 633,  HA_ERR_INDEX_FILE_FULL, IS, "Table fragment hash index has reached maximum possible size" },
+  { 633,  HA_ERR_INDEX_FILE_FULL, IS,
+    "Table fragment hash index has reached maximum possible size" },
   { 640,  DMEC, IS, "Too many hash indexes (should not happen)" },
   { 826,  HA_ERR_RECORD_FILE_FULL, IS, "Too many tables and attributes (increase MaxNoOfAttributes or MaxNoOfTables)" },
   { 827,  HA_ERR_RECORD_FILE_FULL, IS, "Out of memory in Ndb Kernel, table data (increase DataMemory)" },
+  { 889,  HA_ERR_RECORD_FILE_FULL, IS,
+    "Table fragment fixed data reference has reached maximum possible value (specify MAXROWS or increase no of partitions)"},
   { 902,  HA_ERR_RECORD_FILE_FULL, IS, "Out of memory in Ndb Kernel, ordered index data (increase DataMemory)" },
   { 903,  HA_ERR_INDEX_FILE_FULL, IS, "Too many ordered indexes (increase MaxNoOfOrderedIndexes)" },
   { 904,  HA_ERR_INDEX_FILE_FULL, IS, "Out of fragment records (increase MaxNoOfOrderedIndexes)" },
   { 905,  DMEC, IS, "Out of attribute records (increase MaxNoOfAttributes)" },
   { 1601, HA_ERR_RECORD_FILE_FULL, IS, "Out extents, tablespace full" },
   { 1602, DMEC, IS,"No datafile in tablespace" },
+  { 1603, HA_ERR_RECORD_FILE_FULL, IS,
+    "Table fragment fixed data reference has reached maximum possible value (specify MAXROWS or increase no of partitions)"},
 
   /**
    * TimeoutExpired 
@@ -238,6 +251,7 @@ ErrorBundle ErrorCodes[] = {
    * Internal Temporary
    */
   { 702,  DMEC, IT, "Request to non-master" },
+  { 787,  DMEC, IT, "Schema transaction aborted" },
   
   /**
    * Internal errors
@@ -260,9 +274,15 @@ ErrorBundle ErrorCodes[] = {
   { 278,  DMEC, IE, "278" },
   { 287,  DMEC, IE, "Index corrupted" },
   { 290,  DMEC, IE, "Corrupt key in TC, unable to xfrm" },
+  { 293,  DMEC, IE, "Inconsistent trigger state in TC block" },
+  { 292,  DMEC, IE, "Inconsistent index state in TC block" },
   { 631,  DMEC, IE, "631" },
   { 632,  DMEC, IE, "632" },
   { 706,  DMEC, IE, "Inconsistency during table creation" },
+  { 781,  DMEC, IE, "Invalid schema transaction key from NDB API" },
+  { 782,  DMEC, IE, "Invalid schema transaction id from NDB API" },
+  { 784,  DMEC, TR, "Invalid schema transaction state" },
+  { 788,  DMEC, TR, "Missing schema operation at takeover of schema transaction" },
   { 809,  DMEC, IE, "809" },
   { 812,  DMEC, IE, "812" },
   { 833,  DMEC, IE, "833" },
@@ -292,16 +312,25 @@ ErrorBundle ErrorCodes[] = {
   { 4349, DMEC, IE, "Inconsistency detected at index usage" },
   { 4350, DMEC, IE, "Transaction already aborted" },
   { 4351, DMEC, TO, "Timeout/deadlock during index build" },
+  { 294,  DMEC, IE, "Unlocked operation has out of range index" },
+  { 295,  DMEC, IE, "Unlocked operation has invalid state" },
+  { 298,  DMEC, IE, "Invalid distribution key" },
+  { 416,  DMEC, IE, "Bad state handling unlock request" },
 
   /**
    * Application error
    */
   { 281,  HA_ERR_NO_CONNECTION, AE, "Operation not allowed due to cluster shutdown in progress" },
   { 299,  DMEC, AE, "Operation not allowed or aborted due to single user mode" },
-  { 763,  DMEC, AE, "Alter table requires cluster nodes to have exact same version" },
+  { 261,  DMEC, AE,
+    "DML count in transaction exceeds config parameter MaxDMLOperationsPerTransaction" },
+  { 763,  DMEC, AE, "DDL is not supported with mixed data-node versions" },
   { 823,  DMEC, AE, "Too much attrinfo from application in tuple manager" },
   { 829,  DMEC, AE, "Corrupt data received for insert/update" },
   { 831,  DMEC, AE, "Too many nullable/bitfields in table definition" },
+  { 850,  DMEC, AE, "Too long or too short default value"},
+  { 851,  DMEC, AE, "Maximum 8052 bytes of FIXED columns supported"
+    ", use varchar or COLUMN_FORMAT DYNMIC instead" },
   { 876,  DMEC, AE, "876" },
   { 877,  DMEC, AE, "877" },
   { 878,  DMEC, AE, "878" },
@@ -315,6 +344,12 @@ ErrorBundle ErrorCodes[] = {
   { 1233, DMEC, AE, "Table read-only" },
   { 4256, DMEC, AE, "Must call Ndb::init() before this function" },
   { 4257, DMEC, AE, "Tried to read too much - too many getValue calls" },
+  { 320,  DMEC, AE, "Invalid no of nodes specified for new nodegroup" },
+  { 321,  DMEC, AE, "Invalid nodegroup id" },
+  { 322,  DMEC, AE, "Invalid node(s) specified for new nodegroup, node already in nodegroup" },
+  { 323,  DMEC, AE, "Invalid nodegroup id, nodegroup already existing" },
+  { 324,  DMEC, AE, "Invalid node(s) specified for new nodegroup, no node in nodegroup is started" },
+  { 417,  DMEC, AE, "Bad operation reference - double unlock" },
 
   /** 
    * Scan application errors
@@ -363,6 +398,7 @@ ErrorBundle ErrorCodes[] = {
   /**
    * SchemaError
    */
+  { 306,  DMEC, IE, "Out of fragment records in DIH" },
   { 311,  DMEC, AE, "Undefined partition used in setPartitionId" },
   { 703,  DMEC, SE, "Invalid table format" },
   { 704,  DMEC, SE, "Attribute name too long" },
@@ -387,6 +423,7 @@ ErrorBundle ErrorCodes[] = {
   { 749,  HA_WRONG_CREATE_OPTION, IE, "Primary Table in wrong state" },
   { 779,  HA_WRONG_CREATE_OPTION, SE, "Invalid undo buffer size" },
   { 791,  HA_WRONG_CREATE_OPTION, SE, "Too many total bits in bitfields" },
+  { 795,  DMEC, IE, "Out of LongMessageBuffer in DICT" },
   { 764,  HA_WRONG_CREATE_OPTION, SE, "Invalid extent size" },
   { 789,  HA_WRONG_CREATE_OPTION, AE, "Logfile group not found" },
   { 765,  DMEC, SE, "Out of filegroup records" },
@@ -409,6 +446,8 @@ ErrorBundle ErrorCodes[] = {
   { 769,  DMEC, SE, "Drop undofile not supported, drop logfile group instead" },
   { 770,  DMEC, SE, "Cant drop file, file is used" },
   { 774,  DMEC, SE, "Invalid schema object for drop" },
+  { 790,  HA_WRONG_CREATE_OPTION, SE, "Invalid hashmap" },
+  { 793,  DMEC, AE, "Object definition too big" },
   { 241,  HA_ERR_TABLE_DEF_CHANGED, SE, "Invalid schema object version" },
   { 283,  HA_ERR_NO_SUCH_TABLE, SE, "Table is being dropped" },
   { 284,  HA_ERR_TABLE_DEF_CHANGED, SE, "Table not defined in transaction coordinator" },
@@ -419,6 +458,13 @@ ErrorBundle ErrorCodes[] = {
   { 908,  DMEC, IS, "Invalid ordered index tree node size" },
   { 909,  DMEC, IE, "No free index scan op" },
   { 910, HA_ERR_NO_SUCH_TABLE, SE, "Index is being dropped" },
+  { 913,  DMEC, AE, "Invalid index for index state update" },
+  { 914,  DMEC, IE, "Invalid index stats request" },
+  { 915,  DMEC, TR, "No free index stats op" },
+  { 916,  DMEC, IE, "Invalid index stats sys tables" },
+  { 917,  DMEC, IE, "Invalid index stats sys tables data" },
+  { 918,  DMEC, TR, "Cannot prepare index stats update" },
+  { 919,  DMEC, TR, "Cannot execute index stats update" },
   { 1224, HA_WRONG_CREATE_OPTION, SE, "Too many fragments" },
   { 1225, DMEC, SE, "Table not defined in local query handler" },
   { 1226, DMEC, SE, "Table is being dropped" },
@@ -449,7 +495,11 @@ ErrorBundle ErrorCodes[] = {
   { 776,  DMEC, AE, "Index created on temporary table must itself be temporary" },
   { 777,  DMEC, AE, "Cannot create a temporary index on a non-temporary table" },
   { 778,  DMEC, AE, "A temporary table or index must be specified as not logging" },
-  
+  { 786,  DMEC, NR, "Schema transaction aborted due to node-failure" },
+  { 792,  DMEC, SE, "Default value for primary key column not supported" },
+  { 794,  DMEC, AE, "Schema feature requires data node upgrade" },
+  { 796,  DMEC, SE, "Out of schema transaction memory" },
+
   /**
    * FunctionNotImplemented
    */
@@ -461,7 +511,7 @@ ErrorBundle ErrorCodes[] = {
 
   { 1300, DMEC, IE, "Undefined error" },
   { 1301, DMEC, IE, "Backup issued to not master (reissue command to master)" },
-  { 1302, DMEC, IE, "Out of backup record" },
+  { 1302, DMEC, AE, "A backup is already running" },
   { 1303, DMEC, IS, "Out of resources" },
   { 1304, DMEC, IE, "Sequence failure" },
   { 1305, DMEC, IE, "Backup definition not implemented" },
@@ -496,6 +546,22 @@ ErrorBundle ErrorCodes[] = {
   { 1702, DMEC, AE, "Node already connected" },
   { 1703, DMEC, IT, "Node failure handling not completed" },
   { 1704, DMEC, AE, "Node type mismatch" },
+
+  /*
+   * Index stats error codes
+   */
+  { 4714, DMEC, AE, "Index stats sys tables " NDB_INDEX_STAT_PREFIX " do not exist" },
+  { 4715, DMEC, AE, "Index stats for specified index do not exist" },
+  { 4716, DMEC, AE, "Index stats methods usage error" },
+  { 4717, DMEC, AE, "Index stats cannot allocate memory" },
+  { 4718, DMEC, IE, "Index stats samples data or memory cache is invalid" },
+  { 4719, DMEC, IE, "Index stats internal error" },
+  { 4720, DMEC, AE, "Index stats sys tables " NDB_INDEX_STAT_PREFIX " partly missing or invalid" },
+  { 4721, DMEC, IE, "Mysqld: index stats thread not open for requests" },
+  { 4722, DMEC, IE, "Mysqld: index stats entry unexpectedly not found" },
+  { 4723, DMEC, AE, "Mysqld: index stats request ignored due to recent error" },
+  { 4724, DMEC, AE, "Mysqld: index stats request aborted by stats thread" },
+  { 4725, DMEC, AE, "Index stats were deleted by another process" },
   
   /**
    * Still uncategorized
@@ -521,6 +587,8 @@ ErrorBundle ErrorCodes[] = {
   { 1425, DMEC, SE, "Subscription being defined...while trying to stop subscriber" },
   { 1426, DMEC, SE, "No such subscriber" },
   { 1427, DMEC, NR, "Api node died, when SUB_START_REQ reached node "},
+  { 1428, DMEC, IE, "No replica to scan on this node (internal index stats error)" },
+  { 1429, DMEC, IE, "Subscriber node undefined in SubStartReq (config change??)" },
 
   { 4004, DMEC, AE, "Attribute name or id not found in the table" },
   
@@ -535,6 +603,7 @@ ErrorBundle ErrorCodes[] = {
   { 4116, DMEC, AE, "Operation was not defined correctly, probably missing a key" },
   { 4117, DMEC, AE, "Could not start transporter, configuration error"}, 
   { 4118, DMEC, AE, "Parameter error in API call" },
+  { 4120, DMEC, AE, "Scan already complete" },
   { 4300, DMEC, AE, "Tuple Key Type not correct" },
   { 4301, DMEC, AE, "Fragment Type not correct" },
   { 4302, DMEC, AE, "Minimum Load Factor not correct" },
@@ -568,6 +637,8 @@ ErrorBundle ErrorCodes[] = {
   { 4400, DMEC, AE, "Status Error in NdbSchemaCon" },
   { 4401, DMEC, AE, "Only one schema operation per schema transaction" },
   { 4402, DMEC, AE, "No schema operation defined before calling execute" },
+  { 4410, DMEC, AE, "Schema transaction is already started" },
+  { 4411, DMEC, AE, "Schema transaction not possible until upgrade complete" },
 
   { 4501, DMEC, AE, "Insert in hash table failed when getting table information from Ndb" },
   { 4502, DMEC, AE, "GetValue not allowed in Update operation" },
@@ -604,9 +675,16 @@ ErrorBundle ErrorCodes[] = {
   { 4543, DMEC, AE, "Duplicate partitioning information supplied" },
   { 4544, DMEC, AE, "Wrong partitionInfo type for table" },
   { 4545, DMEC, AE, "Invalid or Unsupported PartitionInfo structure" },
-  /* 4546 used in later releases */
+  { 4546, DMEC, AE, "Explicit partitioning info not allowed for table and operation" },
   { 4547, DMEC, AE, "RecordSpecification has overlapping offsets" },
   { 4548, DMEC, AE, "RecordSpecification has too many elements" },
+  { 4549, DMEC, AE, "getLockHandle only supported for primary key read with a lock" },
+  { 4550, DMEC, AE, "Cannot releaseLockHandle until operation executed" },
+  { 4551, DMEC, AE, "NdbLockHandle already released" },
+  { 4552, DMEC, AE, "NdbLockHandle does not belong to transaction" },
+  { 4553, DMEC, AE, "NdbLockHandle original operation not executed successfully" },
+  { 4554, DMEC, AE, "NdbBlob can only be closed from Active state" },
+  { 4555, DMEC, AE, "NdbBlob cannot be closed with pending operations" },
 
   { 4200, DMEC, AE, "Status Error when defining an operation" },
   { 4201, DMEC, AE, "Variable Arrays not yet supported" },
@@ -708,6 +786,60 @@ ErrorBundle ErrorCodes[] = {
   { 2810, DMEC, TR, "No space left on the device" },
   { 2811, DMEC, TR, "Error with file permissions, please check file system" },
   { 2815, DMEC, TR, "Error in reading files, please check file system" },
+  {  920, DMEC, AE, "Row operation defined after refreshTuple()" },
+
+  /**
+   * NdbQueryBuilder API errors
+   */
+  { QRY_REQ_ARG_IS_NULL, DMEC, AE, 
+    "Required argument is NULL" },
+  { QRY_TOO_FEW_KEY_VALUES, DMEC, AE, 
+    "All required 'key' values was not specified" },
+  { QRY_TOO_MANY_KEY_VALUES, DMEC, AE,
+    "Too many 'key' or 'bound' values was specified" },
+  { QRY_OPERAND_HAS_WRONG_TYPE, DMEC, AE, 
+    "Incompatible datatype specified in operand argument" },
+  { QRY_CHAR_OPERAND_TRUNCATED, DMEC, AE, 
+    "Character operand was right truncated" },
+  { QRY_NUM_OPERAND_RANGE, DMEC, AE, 
+    "Numeric operand out of range" },
+  { QRY_MULTIPLE_PARENTS, DMEC, AE, 
+    "Multiple 'parents' specified in linkedValues for this operation" },
+  { QRY_UNKNOWN_PARENT, DMEC, AE,
+    "Unknown 'parent' specified in linkedValue" },
+  { QRY_UNRELATED_INDEX, DMEC, AE, 
+    "Specified 'index' does not belong to specified 'table'" },
+  { QRY_WRONG_INDEX_TYPE, DMEC, AE, 
+    "Wrong type of index specified for this operation"},
+  { QRY_OPERAND_ALREADY_BOUND, DMEC, AE, 
+    "Can't use same operand value to specify different column values" },
+  { QRY_DEFINITION_TOO_LARGE, DMEC, AE, 
+    "Query definition too large." },
+  { QRY_RESULT_ROW_ALREADY_DEFINED, DMEC, AE, 
+    "Result row already defined for NdbQueryOperation."},
+  { QRY_HAS_ZERO_OPERATIONS, DMEC, AE, 
+    "Query defintion should have at least one operation."},
+  { QRY_IN_ERROR_STATE, DMEC, AE, 
+    "A previous query operation failed, which you missed to catch."},
+  { QRY_ILLEGAL_STATE, DMEC, AE, 
+    "Query is in illegal state for this operation."},
+  { QRY_WRONG_OPERATION_TYPE, DMEC, AE, 
+    "This method cannot be invoked on this type of operation (lookup/scan/"
+    "index scan)."},
+  { QRY_SCAN_ORDER_ALREADY_SET, DMEC, AE, 
+    "Index scan order was already set in query definition."},
+  { QRY_PARAMETER_HAS_WRONG_TYPE, DMEC, AE, 
+    "Parameter value has an incompatible datatype" },
+  { QRY_CHAR_PARAMETER_TRUNCATED, DMEC, AE, 
+    "Character Parameter was right truncated" },
+  { QRY_MULTIPLE_SCAN_SORTED, DMEC, AE, 
+    "Query with multiple scans may not be sorted." },
+  { QRY_SEQUENTIAL_SCAN_SORTED, DMEC, AE, 
+    "Parallelism cannot be restricted for sorted scans." },
+  { QRY_BATCH_SIZE_TOO_SMALL, DMEC, AE, 
+    "Batch size for sub scan cannot be smaller than number of fragments." },
+  { QRY_EMPTY_PROJECTION, DMEC, AE,
+    "Query has operation with empty projection." },
 
   { NO_CONTACT_WITH_PROCESS, DMEC, AE,
     "No contact with the process (dead ?)."},
@@ -859,17 +991,17 @@ int ndb_error_string(int err_no, char *str, int size)
   int len;
 
   assert(size > 1);
-  if(size <= 1) 
+  if(size <= 1)
     return 0;
+
   error.code = err_no;
   ndberror_update(&error);
 
-  len =
-    my_snprintf(str, size-1, "%s: %s: %s", error.message,
+  len = (int)my_snprintf(str, size-1, "%s: %s: %s", error.message,
 		ndberror_status_message(error.status),
 		ndberror_classification_message(error.classification));
   str[size-1]= '\0';
-  
+
   if (error.classification != UE)
     return len;
   return -len;
