@@ -63,20 +63,11 @@ int runDropAllTablesExceptTestTable(NDBT_Context* ctx, NDBT_Step* step){
       return NDBT_ProgramExit(NDBT_FAILED);
     }
 
-    // Don't drop test table
-    if (strcmp(tab->getName(), ctx->getTab()->getName()) == 0){
-      continue;
-    }
-	
-    int res = GETNDB(step)->getDictionary()->dropTable(tab->getName());
-    if(res == -1){
-      return NDBT_FAILED;
-    }
+    GETNDB(step)->getDictionary()->dropTable(tab->getName());
   }
   return NDBT_OK;
 }
 
-
 int runLoadAllTables(NDBT_Context* ctx, NDBT_Step* step){
   
   int records = ctx->getNumRecords();
@@ -306,12 +297,27 @@ int runScanRead(NDBT_Context* ctx, NDBT_Step* step){
   int records = ctx->getProperty("Rows", ctx->getNumRecords());
   int parallelism = ctx->getProperty("Parallelism", 240);
   int abort = ctx->getProperty("AbortProb", 5);
+  int tupscan = ctx->getProperty("TupScan", (Uint32)0);
+  int lockmode = ctx->getProperty("LockMode", NdbOperation::LM_CommittedRead);
 
   int i = 0;
   HugoTransactions hugoTrans(*ctx->getTab());
   while (i<loops && !ctx->isTestStopped()) {
     g_info << i << ": ";
-    if (hugoTrans.scanReadRecords(GETNDB(step), records, abort, parallelism) != 0){
+
+    int scan_flags = 0;
+    if (tupscan == 1)
+    {
+      scan_flags |= NdbScanOperation::SF_TupScan;
+      if (hugoTrans.scanReadRecords(GETNDB(step), records, abort, parallelism,
+                                    NdbOperation::LockMode(lockmode),
+                                    scan_flags) != 0)
+        return NDBT_FAILED;
+    }
+    else if (hugoTrans.scanReadRecords(GETNDB(step), records, abort,
+                                       parallelism,
+                                       NdbOperation::LockMode(lockmode)) != 0)
+    {
       return NDBT_FAILED;
     }
     i++;
@@ -360,11 +366,13 @@ int runRandScanRead(NDBT_Context* ctx, NDBT_Step* step){
 
 int runScanReadIndex(NDBT_Context* ctx, NDBT_Step* step){
   int loops = ctx->getNumLoops();
-  int records = ctx->getNumRecords();
+  int records = ctx->getProperty("Rows", ctx->getNumRecords());
   int parallelism = ctx->getProperty("Parallelism", 240);
   int abort = ctx->getProperty("AbortProb", 5);
-  const NdbDictionary::Index * pIdx = 
-    GETNDB(step)->getDictionary()->getIndex(orderedPkIdxName, 
+  int lockmode = ctx->getProperty("LockMode", NdbOperation::LM_CommittedRead);
+  int rand_mode = ctx->getProperty("RandScanOptions", Uint32(1));
+  const NdbDictionary::Index * pIdx =
+    GETNDB(step)->getDictionary()->getIndex(orderedPkIdxName,
 					    ctx->getTab()->getName());
 
   int i = 0;
@@ -373,11 +381,17 @@ int runScanReadIndex(NDBT_Context* ctx, NDBT_Step* step){
     g_info << i << ": ";
     bool sort = (rand() % 100) > 50 ? true : false;
     bool desc = (rand() % 100) > 50 ? true : false;
+    NdbOperation::LockMode lm = (NdbOperation::LockMode)(rand() % 3);
     desc = false;       // random causes too many deadlocks
+    if (rand_mode == 0)
+    {
+      sort = false;
+      desc = false;
+      lm = (NdbOperation::LockMode)lockmode;
+    }
     int scan_flags =
       (NdbScanOperation::SF_OrderBy & -(int)sort) |
       (NdbScanOperation::SF_Descending & -(int)desc);
-    NdbOperation::LockMode lm = (NdbOperation::LockMode)(rand() % 3);
     if (hugoTrans.scanReadRecords(GETNDB(step), pIdx,
 				  records, abort, parallelism,
 				  lm,
@@ -654,7 +668,7 @@ int runScanUpdate(NDBT_Context* ctx, NDBT_Step* step){
 }
 
 int runScanUpdateUntilStopped(NDBT_Context* ctx, NDBT_Step* step){
-  int records = ctx->getNumRecords();
+  //int records = ctx->getNumRecords();
   int i = 0;
 
   int parallelism = ctx->getProperty("Parallelism", 240);
@@ -1137,8 +1151,8 @@ runScanParallelism(NDBT_Context* ctx, NDBT_Step* step){
 int
 runScanVariants(NDBT_Context* ctx, NDBT_Step* step)
 {
-  int loops = ctx->getNumLoops();
-  int records = ctx->getNumRecords();
+  //int loops = ctx->getNumLoops();
+  //int records = ctx->getNumRecords();
   Ndb * pNdb = GETNDB(step);
   const NdbDictionary::Table*  pTab = ctx->getTab();
   
@@ -1204,7 +1218,7 @@ runScanVariants(NDBT_Context* ctx, NDBT_Step* step)
 	    }
 	    
 	    int res;
-	    int row = 0;
+	    //int row = 0;
 	    while((res = pOp->nextResult()) == 0);
 	  }
 	  pCon->close();
@@ -1275,7 +1289,7 @@ runBug24447(NDBT_Context* ctx, NDBT_Step* step){
       return NDBT_FAILED;
     }
 
-    for (Uint32 j = 0; i<10; i++)
+    for (Uint32 j = 0; j<10; j++)
     {
       hugoTrans.scanReadRecords(GETNDB(step), records, abort, 0, 
 				NdbOperation::LM_CommittedRead);
@@ -1376,220 +1390,6 @@ finalizeBug42559(NDBT_Context* ctx, NDBT_Step* step){
   return NDBT_OK;
 }
 
-int takeResourceSnapshot(NDBT_Context* ctx, NDBT_Step* step)
-{
-  NdbRestarter restarter;
-  
-  int checksnapshot = DumpStateOrd::TcResourceSnapshot;
-  restarter.dumpStateAllNodes(&checksnapshot, 1);
-
-  /* TODO : Check other block's resources? */
-  return NDBT_OK;
-}
-
-int runScanReadIndexWithBounds(NDBT_Context* ctx, NDBT_Step* step){
-  int loops = ctx->getNumLoops();
-  int records = ctx->getNumRecords();
-  int numRanges = ctx->getProperty("NumRanges", 1);
-  int maxRunSecs = ctx->getProperty("MaxRunSecs", 60);
-  int maxRetries = ctx->getProperty("MaxRetries", 1000000);
-  
-  const NdbDictionary::Index * pIdx = 
-    GETNDB(step)->getDictionary()->getIndex(orderedPkIdxName, 
-					    ctx->getTab()->getName());
-
-  int i = 0;
-  HugoCalculator calc(*ctx->getTab());
-  NDBT_ResultRow row (*ctx->getTab());
-  Ndb* ndb = GETNDB(step);
-
-  Uint64 start = NdbTick_CurrentMillisecond();
-  Uint64 end = start + (1000*maxRunSecs);
-  int retries = 0;
-
-  /* Here we run an ordered index scan, with a bound.
-   * There are numRanges sub-scans with the same bound
-   * This is done to use up some KeyInfo, and expose bugs in that area
-   * If we run many of these in parallel we may exhaust the available KeyInfo storage,
-   * which may expose some bugs.
-   */
-  while (pIdx &&
-         i<loops && 
-         !ctx->isTestStopped() &&
-         NdbTick_CurrentMillisecond() < end) {
-    g_info << "Step " << step->getStepNo() 
-           << "Loop : " << i << ": ";
-    
-    /* Use specific-partition variant of startTransaction to ensure a single
-     * TC node is used
-     */
-    NdbTransaction* trans = ndb->startTransaction(ctx->getTab(), Uint32(0));
-    if (trans == NULL)
-    {
-      g_err << "Transaction start failed " << ndb->getNdbError() << endl;
-      return NDBT_FAILED;
-    }
-
-    NdbIndexScanOperation* iso = trans->getNdbIndexScanOperation(pIdx->getName(), 
-                                                                 ctx->getTab()->getName());
-    if (iso == NULL)
-    {
-      g_err << "Error obtaining IndexScanOperation : " << trans->getNdbError() << endl;
-      trans->close();
-      return NDBT_FAILED;
-    }
-
-    if (iso->readTuples(NdbOperation::LM_CommittedRead, 
-                        (NdbScanOperation::SF_OrderBy |
-                         NdbScanOperation::SF_ReadRangeNo |
-                         NdbScanOperation::SF_MultiRange), 
-                        0) != 0)
-    {
-      g_err << "Error calling readTuples : " << iso->getNdbError() << endl;
-      trans->close();
-      return NDBT_FAILED;
-    }
-
-    for (int range = 0; range < numRanges; range++)
-    {
-      /* Now define a bound... */
-      for (Uint32 k=0; k<pIdx->getNoOfColumns(); k++)
-      {
-        const NdbDictionary::Column* idxCol = pIdx->getColumn(k);
-        const char* colName = idxCol->getName();
-        /* Lower bound of <= NULL should return all rows */
-        if (iso->setBound(colName, NdbIndexScanOperation::BoundLE, NULL) != 0)
-        {
-          g_err << "Error setting bound for column %s. " 
-                << iso->getNdbError() << endl;
-          trans->close();
-          return NDBT_FAILED;
-        }
-      }
-      
-      if (iso->end_of_bound(range) != 0)
-      {
-        g_err << "Error closing range " << range << endl;
-        g_err << iso->getNdbError() << endl;
-        return NDBT_FAILED;
-      }
-    }
-    
-    const NdbDictionary::Table& tab= *ctx->getTab();
-
-    /* Now request all columns in result projection */
-    for (int a=0; a<tab.getNoOfColumns(); a++){
-      if((row.attributeStore(a) = 
-	  iso->getValue(tab.getColumn(a)->getName())) == 0) {
-	g_err << "Error defining read value " << trans->getNdbError() << endl;
-        trans->close();
-	return NDBT_FAILED;
-      }
-    }
-          
-    /* Ready to go... */
-    trans->execute(NoCommit, AbortOnError);
-
-    if (trans->getNdbError().code != 0)
-    {
-      if (trans->getNdbError().code == 218)
-      {
-        /* Out of KeyInfo buffers in TC - that's ok, let's try again */
-        trans->close();
-        if (retries++ < maxRetries)
-        {
-          g_err << "Step " << step->getStepNo()
-                << " TC out of Keyinfo buffers (218) - retrying" << endl;
-          continue;
-        }
-      }
-
-      g_err << "Error on execution : " << trans->getNdbError() << endl;
-      trans->close();
-      return NDBT_FAILED;
-    }
-    
-    int eof;
-    int rows = 0;
-    while ((eof = iso->nextResult(true)) == 0)
-    {
-      rows++;
-      if (calc.verifyRowValues(&row) != 0)
-      {
-        g_err << "Verification failed." << endl;
-        trans->close();
-        return NDBT_FAILED;
-      }
-
-#ifdef BUG_14388257_FIXED
-      int rangeNum = (rows -1) / records;
-      if (iso->get_range_no() != rangeNum)
-      {
-        g_err << "Expected row " << rows 
-              << " to be in range " << rangeNum
-              << " but it reports range num " << iso->get_range_no()
-              << " : " << row
-              << endl;
-        return NDBT_FAILED;
-      }
-#endif
-
-      //g_err << row << endl;
-    }
-
-    if (eof != 1)
-    {
-      g_err << "nextResult() returned " << eof << endl;
-      g_err << "Scan error : " << iso->getNdbError() << endl;
-
-      if (iso->getNdbError().status == NdbError::TemporaryError)
-      {
-        if (retries++ < maxRetries)
-        {
-          g_err << "Step "
-                << step->getStepNo()
-                << "  Temporary, retrying on iteration " 
-                << i << " rows so far : " << rows << endl;
-          trans->close();
-          NdbSleep_MilliSleep(2500);
-          continue;
-        }
-      }
-
-      trans->close();
-      return NDBT_FAILED;
-    }
-    
-    g_err << "Read " << rows << " rows." << endl;
-    
-    if (records != 0 && rows != (numRanges * records))
-    {
-      g_err << "Expected " << records << " rows"
-            << ", read " << rows << endl;
-#ifdef BUG_14388257_FIXED
-      trans->close();
-      assert(false);
-      return NDBT_FAILED;
-#endif
-    }
-
-    trans->close();
-    i++;
-  }
-  return NDBT_OK;
-}
-
-int checkResourceSnapshot(NDBT_Context* ctx, NDBT_Step* step)
-{
-  NdbRestarter restarter;
-  
-  int checksnapshot = DumpStateOrd::TcResourceCheckLeak;
-  restarter.dumpStateAllNodes(&checksnapshot, 1);
-
-  /* TODO : Check other block's resources? */
-  return NDBT_OK;
-}
-
 
 int
 runBug54945(NDBT_Context* ctx, NDBT_Step* step)
@@ -1746,17 +1546,142 @@ runCloseRefresh(NDBT_Context* ctx, NDBT_Step* step)
   return NDBT_OK;
 }
 
-// An 'assert' that is always executed, so that 'cond' may have side effects.
-#define require(cond) \
-  do \
-  { \
-    if (!(cond)) \
-    { \
-      g_err << "require(" << #cond << ") failed at " << __FILE__ << ":" \
-        << __LINE__ << endl; \
-      abort(); \
-    } \
-  }while(false)
+#define CHK_RET_FAILED(x) if (!(x)) { ndbout_c("Failed on line: %u", __LINE__); return NDBT_FAILED; }
+
+int
+runMixedDML(NDBT_Context* ctx, NDBT_Step* step)
+{
+  Ndb* pNdb = GETNDB(step);
+  const NdbDictionary::Table* pTab = ctx->getTab();
+
+  unsigned seed = (unsigned)NdbTick_CurrentMillisecond();
+
+  const int rows = ctx->getNumRecords();
+  const int loops = 10 * ctx->getNumLoops();
+  const int until_stopped = ctx->getProperty("UntilStopped");
+  const int batch = ctx->getProperty("Batch", Uint32(50));
+
+  const NdbRecord * pRowRecord = pTab->getDefaultRecord();
+  CHK_RET_FAILED(pRowRecord != 0);
+
+  const Uint32 len = NdbDictionary::getRecordRowLength(pRowRecord);
+  Uint8 * pRow = new Uint8[len];
+
+  int count_ok = 0;
+  int count_failed = 0;
+  for (int i = 0; i < loops || (until_stopped && !ctx->isTestStopped()); i++)
+  {
+    NdbTransaction* pTrans = pNdb->startTransaction();
+    CHK_RET_FAILED(pTrans != 0);
+
+    int lastrow = 0;
+    int result = 0;
+    for (int rowNo = 0; rowNo < batch; rowNo++)
+    {
+      int left = rows - lastrow;
+      int rowId = lastrow;
+      if (left)
+      {
+        rowId += ndb_rand_r(&seed) % (left / 10 + 1);
+      }
+      else
+      {
+        break;
+      }
+      lastrow = rowId;
+
+      bzero(pRow, len);
+
+      HugoCalculator calc(* pTab);
+      calc.setValues(pRow, pRowRecord, rowId, rand());
+
+      NdbOperation::OperationOptions opts;
+      bzero(&opts, sizeof(opts));
+
+      const NdbOperation* pOp = 0;
+      switch(ndb_rand_r(&seed) % 3){
+      case 0:
+        pOp = pTrans->writeTuple(pRowRecord, (char*)pRow,
+                                 pRowRecord, (char*)pRow,
+                                 0,
+                                 &opts,
+                                 sizeof(opts));
+        break;
+      case 1:
+        pOp = pTrans->deleteTuple(pRowRecord, (char*)pRow,
+                                  pRowRecord, (char*)pRow,
+                                  0,
+                                  &opts,
+                                  sizeof(opts));
+        break;
+      case 2:
+        pOp = pTrans->updateTuple(pRowRecord, (char*)pRow,
+                                  pRowRecord, (char*)pRow,
+                                  0,
+                                  &opts,
+                                  sizeof(opts));
+        break;
+      }
+      CHK_RET_FAILED(pOp != 0);
+      result = pTrans->execute(NoCommit, AO_IgnoreError);
+      if (result != 0)
+      {
+        goto found_error;
+      }
+    }
+
+    result = pTrans->execute(Commit, AO_IgnoreError);
+    if (result != 0)
+    {
+  found_error:
+      count_failed++;
+      NdbError err = pTrans->getNdbError();
+      ndbout << err << endl;
+      CHK_RET_FAILED(err.status == NdbError::TemporaryError ||
+                     err.classification == NdbError::NoDataFound ||
+                     err.classification == NdbError::ConstraintViolation);
+    }
+    else
+    {
+      count_ok++;
+    }
+    pTrans->close();
+  }
+
+  ndbout_c("count_ok: %d count_failed: %d",
+           count_ok, count_failed);
+  delete [] pRow;
+
+  return NDBT_OK;
+}
+
+int
+runBug13394788(NDBT_Context* ctx, NDBT_Step* step)
+{
+  const int loops = ctx->getNumLoops();
+  const int records = ctx->getNumRecords();
+  const NdbDictionary::Index * pIdx =
+    GETNDB(step)->getDictionary()->getIndex(orderedPkIdxName,
+					    ctx->getTab()->getName());
+  HugoTransactions hugoTrans(*ctx->getTab(), pIdx);
+
+  NdbRestarter res;
+  for (int i = 0; i < loops; i++)
+  {
+    res.insertErrorInAllNodes(5074);
+    // this will actually be a mrr scan...
+    int batch = 1 + (rand() % records);
+    // this should be error...
+    hugoTrans.pkReadRecords(GETNDB(step), records, batch);
+
+    // make it should work again...
+    res.insertErrorInAllNodes(0);
+    if (hugoTrans.pkReadRecords(GETNDB(step), records, batch) != 0)
+      return NDBT_FAILED;
+  }
+
+  return NDBT_OK;
+}
 
 /**
  * This is a regression test for bug #11748194 "TRANSACTION OBJECT CREATED 
@@ -1774,7 +1699,7 @@ runExtraNextResult(NDBT_Context* ctx, NDBT_Step* step)
   hugoTrans.loadTable(ndb, 10);
   // Read MaxNoOfConcurrentTransactions configuration value.
   Uint32 maxTrans = 0;
-  NdbConfig conf(ndb->getNodeId());
+  NdbConfig conf;
   require(conf.getProperty(conf.getMasterNodeId(),
                            NODE_TYPE_DB,
                            CFG_DB_NO_TRANSACTIONS,
@@ -1793,7 +1718,6 @@ runExtraNextResult(NDBT_Context* ctx, NDBT_Step* step)
     {
       g_err << "ndb->startTransaction() gave unexpected error : " 
             << ndb->getNdbError() << " in the " << i << "th iteration." <<endl;
-      return NDBT_FAILED;
     }
     
     // Do a random numer of scans in this transaction.
@@ -2019,6 +1943,13 @@ TESTCASE("ScanRead100",
   STEPS(runScanRead, 100);
   FINALIZER(runClearTable);
 }
+TESTCASE("TupScanRead100",
+	 "Verify scan requirement: Scan with 100 simultaneous threads"){
+  TC_PROPERTY("TupScan", 1);
+  INITIALIZER(runLoadTable);
+  STEPS(runScanRead, 100);
+  FINALIZER(runClearTable);
+}
 TESTCASE("Scan-bug8262", 
 	 ""){
   TC_PROPERTY("Rows", 1);
@@ -2393,57 +2324,34 @@ TESTCASE("ScanFragRecExhaust",
   INITIALIZER(runScanReadExhaust);
   FINALIZER(runClearTable);
 }
+TESTCASE("Bug12324191", "")
+{
+  TC_PROPERTY("LockMode", Uint32(NdbOperation::LM_Read));
+  TC_PROPERTY("TupScan", Uint32(1));
+  TC_PROPERTY("Rows", Uint32(0));
+  INITIALIZER(runLoadTable);
+  STEP(runScanRead);
+  STEPS(runMixedDML,10);
+}
+TESTCASE("Bug13394788", "")
+{
+  INITIALIZER(createOrderedPkIndex);
+  INITIALIZER(runLoadTable);
+  STEP(runBug13394788);
+  FINALIZER(createOrderedPkIndex_Drop);
+  FINALIZER(runClearTable);
+}
 TESTCASE("extraNextResultBug11748194",
          "Regression test for bug #11748194")
 {
   INITIALIZER(runExtraNextResult);
 }
-TESTCASE("ScanRealKeyInfoExhaust",
-         "Test behaviour when TC keyinfo buffers exhausted 4real")
-{
-  /* 55 threads, each setting 200 ranges in their keyinfo
-   * For the lightest single column PK case, each range should
-   * use 2 words, 200 ranges = 400 words per scan thread = 
-   * 400/4 = 100 Databuffers used.
-   * 55 threads = 55*100 = 5500 Databuffers which is >
-   * the 4000 statically allocated in 6.3
-   */
-  TC_PROPERTY("NumRanges", 200);
-  TC_PROPERTY("MaxRunSecs", 120);
-  INITIALIZER(createOrderedPkIndex);
-  INITIALIZER(runLoadTable);
-  INITIALIZER(takeResourceSnapshot);
-  STEPS(runScanReadIndexWithBounds,55);
-  FINALIZER(checkResourceSnapshot);
-  FINALIZER(createOrderedPkIndex_Drop);
-  FINALIZER(runClearTable);
-}
-TESTCASE("ScanKeyInfoExhaust",
-         "Test behaviour when TC keyinfo buffers exhausted with error insert")
-{
-  /* Use error insert 8094 to cause keyinfo exhaustion, then run a single scan
-   * with keyinfo to hit the error path
-   */
-  TC_PROPERTY("MaxRunSecs", 10);
-  INITIALIZER(createOrderedPkIndex);
-  INITIALIZER(runLoadTable);
-  INITIALIZER(takeResourceSnapshot);
-  TC_PROPERTY("ErrorCode", 8094);
-  INITIALIZER(runInsertError);
-  STEP(runScanReadIndexWithBounds);
-  FINALIZER(checkResourceSnapshot);
-  FINALIZER(runInsertError);
-  FINALIZER(createOrderedPkIndex_Drop);
-  FINALIZER(runClearTable);
-}
-  
-
-
 NDBT_TESTSUITE_END(testScan);
 
 int main(int argc, const char** argv){
   ndb_init();
-  myRandom48Init(NdbTick_CurrentMillisecond());
+  myRandom48Init((long)NdbTick_CurrentMillisecond());
+  NDBT_TESTSUITE_INSTANCE(testScan);
   return testScan.execute(argc, argv);
 }
 
