@@ -28,6 +28,7 @@
 
 #include "consumer_restore.hpp"
 #include "consumer_printer.hpp"
+#include "../src/ndbapi/NdbDictionaryImpl.hpp"
 
 extern FilteredNdbOut err;
 extern FilteredNdbOut info;
@@ -57,7 +58,7 @@ const char *opt_ndb_table= NULL;
 unsigned int opt_verbose;
 unsigned int opt_hex_format;
 unsigned int opt_progress_frequency;
-unsigned int g_report_next;
+NDB_TICKS g_report_next;
 Vector<BaseString> g_databases;
 Vector<BaseString> g_tables;
 Vector<BaseString> g_include_tables, g_exclude_tables;
@@ -79,8 +80,6 @@ static void save_include_exclude(int optid, char * argument);
 
 static inline void parse_rewrite_database(char * argument);
 
-NDB_STD_OPTS_VARS;
-
 /**
  * print and restore flags
  */
@@ -106,28 +105,12 @@ BaseString g_options("ndb_restore");
 const char *load_default_groups[]= { "mysql_cluster","ndb_restore",0 };
 
 enum ndb_restore_options {
-  OPT_PRINT= NDB_STD_OPTIONS_LAST,
-  OPT_PRINT_DATA,
-  OPT_PRINT_LOG,
-  OPT_PRINT_META,
-  OPT_BACKUP_PATH,
-  OPT_HEX_FORMAT,
-  OPT_FIELDS_ENCLOSED_BY,
-  OPT_FIELDS_TERMINATED_BY,
-  OPT_FIELDS_OPTIONALLY_ENCLOSED_BY,
-  OPT_LINES_TERMINATED_BY,
-  OPT_APPEND,
-  OPT_PROGRESS_FREQUENCY,
-  OPT_NO_BINLOG,
-  OPT_VERBOSE,
+  OPT_VERBOSE = NDB_STD_OPTIONS_LAST,
   OPT_INCLUDE_TABLES,
   OPT_EXCLUDE_TABLES,
   OPT_INCLUDE_DATABASES,
   OPT_EXCLUDE_DATABASES,
-  OPT_REWRITE_DATABASE,
-  OPT_EXCLUDE_MISSING_COLUMNS,
-  OPT_DISABLE_INDEXES,
-  OPT_REBUILD_INDEXES
+  OPT_REWRITE_DATABASE
 };
 static const char *opt_fields_enclosed_by= NULL;
 static const char *opt_fields_terminated_by= NULL;
@@ -146,7 +129,7 @@ static struct my_option my_long_options[] =
 {
   NDB_STD_OPTS("ndb_restore"),
   { "connect", 'c', "same as --connect-string",
-    (uchar**) &opt_connect_str, (uchar**) &opt_connect_str, 0,
+    (uchar**) &opt_ndb_connectstring, (uchar**) &opt_ndb_connectstring, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
   { "nodeid", 'n', "Backup files from node with id",
     (uchar**) &ga_nodeId, (uchar**) &ga_nodeId, 0,
@@ -197,19 +180,19 @@ static struct my_option my_long_options[] =
     "(parallelism can be 1 to 1024)", 
     (uchar**) &ga_nParallelism, (uchar**) &ga_nParallelism, 0,
     GET_INT, REQUIRED_ARG, 128, 1, 1024, 0, 1, 0 },
-  { "print", OPT_PRINT, "Print metadata, data and log to stdout",
+  { "print", NDB_OPT_NOSHORT, "Print metadata, data and log to stdout",
     (uchar**) &_print, (uchar**) &_print, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "print_data", OPT_PRINT_DATA, "Print data to stdout", 
+  { "print_data", NDB_OPT_NOSHORT, "Print data to stdout",
     (uchar**) &_print_data, (uchar**) &_print_data, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "print_meta", OPT_PRINT_META, "Print meta data to stdout",
+  { "print_meta", NDB_OPT_NOSHORT, "Print meta data to stdout",
     (uchar**) &_print_meta, (uchar**) &_print_meta,  0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "print_log", OPT_PRINT_LOG, "Print log to stdout",
+  { "print_log", NDB_OPT_NOSHORT, "Print log to stdout",
     (uchar**) &_print_log, (uchar**) &_print_log,  0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "backup_path", OPT_BACKUP_PATH, "Path to backup files",
+  { "backup_path", NDB_OPT_NOSHORT, "Path to backup files",
     (uchar**) &ga_backupPath, (uchar**) &ga_backupPath, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
   { "dont_ignore_systab_0", 'f',
@@ -222,38 +205,38 @@ static struct my_option my_long_options[] =
     (uchar**) &opt_nodegroup_map_str,
     0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "fields-enclosed-by", OPT_FIELDS_ENCLOSED_BY,
+  { "fields-enclosed-by", NDB_OPT_NOSHORT,
     "Fields are enclosed by ...",
     (uchar**) &opt_fields_enclosed_by, (uchar**) &opt_fields_enclosed_by, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "fields-terminated-by", OPT_FIELDS_TERMINATED_BY,
+  { "fields-terminated-by", NDB_OPT_NOSHORT,
     "Fields are terminated by ...",
     (uchar**) &opt_fields_terminated_by,
     (uchar**) &opt_fields_terminated_by, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "fields-optionally-enclosed-by", OPT_FIELDS_OPTIONALLY_ENCLOSED_BY,
+  { "fields-optionally-enclosed-by", NDB_OPT_NOSHORT,
     "Fields are optionally enclosed by ...",
     (uchar**) &opt_fields_optionally_enclosed_by,
     (uchar**) &opt_fields_optionally_enclosed_by, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "hex", OPT_HEX_FORMAT, "print binary types in hex format", 
+  { "hex", NDB_OPT_NOSHORT, "print binary types in hex format", 
     (uchar**) &opt_hex_format, (uchar**) &opt_hex_format, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { "tab", 'T', "Creates tab separated textfile for each table to "
     "given path. (creates .txt files)",
    (uchar**) &tab_path, (uchar**) &tab_path, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-  { "append", OPT_APPEND, "for --tab append data to file", 
+  { "append", NDB_OPT_NOSHORT, "for --tab append data to file", 
     (uchar**) &opt_append, (uchar**) &opt_append, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "lines-terminated-by", OPT_LINES_TERMINATED_BY, "",
+  { "lines-terminated-by", NDB_OPT_NOSHORT, "",
     (uchar**) &opt_lines_terminated_by, (uchar**) &opt_lines_terminated_by, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "progress-frequency", OPT_PROGRESS_FREQUENCY,
+  { "progress-frequency", NDB_OPT_NOSHORT,
     "Print status uf restore periodically in given seconds", 
     (uchar**) &opt_progress_frequency, (uchar**) &opt_progress_frequency, 0,
     GET_INT, REQUIRED_ARG, 0, 0, 65535, 0, 0, 0 },
-  { "no-binlog", OPT_NO_BINLOG,
+  { "no-binlog", NDB_OPT_NOSHORT,
     "If a mysqld is connected and has binary log, do not log the restored data", 
     (uchar**) &opt_no_binlog, (uchar**) &opt_no_binlog, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
@@ -283,17 +266,17 @@ static struct my_option my_long_options[] =
     "Example: db1.t1,db3.t1",
     (uchar**) &opt_exclude_tables, (uchar**) &opt_exclude_tables, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
-  { "exclude-missing-columns", OPT_EXCLUDE_MISSING_COLUMNS,
+  { "exclude-missing-columns", NDB_OPT_NOSHORT,
     "Ignore columns present in backup but not in database",
     (uchar**) &ga_exclude_missing_columns,
     (uchar**) &ga_exclude_missing_columns, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "disable-indexes", OPT_DISABLE_INDEXES,
+  { "disable-indexes", NDB_OPT_NOSHORT,
     "Disable indexes",
     (uchar**) &ga_disable_indexes,
     (uchar**) &ga_disable_indexes, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "rebuild-indexes", OPT_REBUILD_INDEXES,
+  { "rebuild-indexes", NDB_OPT_NOSHORT,
     "Rebuild indexes",
     (uchar**) &ga_rebuild_indexes,
     (uchar**) &ga_rebuild_indexes, 0,
@@ -344,7 +327,7 @@ static char* analyse_one_map(char *map_str, uint16 *source, uint16 *dest)
   map_str++;
 
   number= strtol(map_str, &end_ptr, 10);
-  if (!end_ptr || number < 0 || number >= UNDEF_NODEGROUP)
+  if (!end_ptr || number < 0 || number >= NDB_UNDEF_NODEGROUP)
   {
     DBUG_RETURN(NULL);
   }
@@ -384,7 +367,7 @@ static void init_nodegroup_map()
   {
     ng_map[i].no_maps= 0;
     for (j= 0; j < MAX_MAPS_PER_NODE_GROUP; j++)
-      ng_map[i].map_array[j]= UNDEF_NODEGROUP;
+      ng_map[i].map_array[j]= NDB_UNDEF_NODEGROUP;
   }
 }
 
@@ -418,17 +401,13 @@ static bool analyse_nodegroup_map(const char *ng_map_str,
 
 static void short_usage_sub(void)
 {
-  printf("Usage: %s [OPTIONS] [<path to backup files>]\n", my_progname);
+  ndb_short_usage_sub("[<path to backup files>]");
 }
 static void usage()
 {
-  short_usage_sub();
-  ndb_std_print_version();
-  print_defaults(MYSQL_CONFIG_NAME,load_default_groups);
-  puts("");
-  my_print_help(my_long_options);
-  my_print_variables(my_long_options);
+  ndb_usage(short_usage_sub, load_default_groups, my_long_options);
 }
+
 static my_bool
 get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 	       char *argument)
@@ -535,10 +514,9 @@ BaseString
 makeExternalTableName(const BaseString &internalName)
 {
    // Make dbname/def/table1 into dbname.table1
-  int idx;
   BaseString externalName;
   
-  idx = internalName.indexOf('/');
+  ssize_t idx = internalName.indexOf('/');
   externalName = internalName.substr(0,idx);
   externalName.append(".");
   externalName.append(internalName.substr(idx + SCHEMA_NAME_SIZE,
@@ -557,6 +535,9 @@ readArguments(int *pargc, char*** pargv)
   init_nodegroup_map();
   load_defaults("my",load_default_groups,pargc,pargv);
   debug << "handle_options" << endl;
+
+  ndb_opt_set_usage_funcs(short_usage_sub, usage);
+
   if (handle_options(pargc, pargv, my_long_options, get_one_option))
   {
     exit(NDBT_ProgramExit(NDBT_WRONGARGS));
@@ -596,7 +577,9 @@ o verify nodegroup mapping
   if (g_printer == NULL)
     return false;
 
-  BackupRestore* restore = new BackupRestore(opt_nodegroup_map,
+  BackupRestore* restore = new BackupRestore(opt_ndb_connectstring,
+                                             opt_ndb_nodeid,
+                                             opt_nodegroup_map,
                                              opt_nodegroup_map_len,
                                              ga_nParallelism);
   if (restore == NULL) 
@@ -1002,12 +985,11 @@ static bool check_include_exclude(BaseString database, BaseString table)
 static inline bool
 checkDoRestore(const TableS* table)
 {
-  int idx;
   bool ret = true;
   BaseString db, tbl;
   
   tbl.assign(getTableName(table));
-  idx = tbl.indexOf('/');
+  ssize_t idx = tbl.indexOf('/');
   db = tbl.substr(0, idx);
   
   /*
@@ -1081,7 +1063,6 @@ free_data_callback()
     g_consumers[i]->tuple_free();
 }
 
-const char * g_connect_string = 0;
 static void exitHandler(int code)
 {
   NDBT_ProgramExit(code);
@@ -1102,7 +1083,7 @@ static int check_progress()
   if (!opt_progress_frequency)
     return 0;
 
-  Uint64 now = NdbTick_CurrentMillisecond() / 1000;
+  NDB_TICKS now = NdbTick_CurrentMillisecond() / 1000;
   
   if (now  >= g_report_next)
   {
@@ -1186,8 +1167,6 @@ main(int argc, char** argv)
   if (ga_skip_broken_objects)
     g_options.append(" --skip-broken-objects");
 
-  g_connect_string = opt_connect_str;
-
   init_progress();
 
   /**
@@ -1218,7 +1197,7 @@ main(int argc, char** argv)
   }
 
   info << endl;
-  
+
   /**
    * check wheater we can restore the backup (right version).
    */
@@ -1243,6 +1222,8 @@ main(int argc, char** argv)
 
   debug << "Load content" << endl;
   int res  = metaData.loadContent();
+
+  info << "Stop GCP of Backup: " << metaData.getStopGCP() << endl;
   
   if (res == 0)
   {
